﻿(function ($) {
    // create jqgl as a namespace
    window.jqgl = {};
    // create the utilities as a singleton
    jqgl.utilities = {
        toPixels: function (canvasMeasure, shapeMeasure) {
            if (typeof shapeMeasure === "string") {
                var value = parseInt(shapeMeasure.match(/\d+/));
                if (shapeMeasure.match(/%$/)) {
                    return (canvasMeasure * value / 100);
                } else {
                    return value;
                }
            } else return shapeMeasure;
        },
        addRenderer: function (name, renderer) {
            console.info("adding renderer: " + name);
            if ($.isFunction(renderer.init))
                renderer.init();
        },
        registerNamespace: function (ns) {
            var _nsParts = ns.split("."), _root = window;
            for (var len = _nsParts.length, i = 0; i < len; i++) {
                if (_root[_nsParts[i]] === undefined) {
                    _root[_nsParts[i]] = {};
                }
                _root = _root[_nsParts[i]];
            }
            return _root;
        }
    };
    jqgl.baseImplementation = function () { };
    jqgl.baseImplementation.prototype = {
        basePath: $("script[src$='jqgl.base.js']").attr("src").replace("jqgl.base.js", ""),
        renderers: ["canvas", "svg", "vml"],
        _renderers: {},
        selectorPrefix: "*",
        types: {},
        methods: {}
    };
    jqgl.widget = function () { };
    jqgl.widget.prototype = {
        options: {},
        _create: function (settings, shapes) {
            var self = this;

            // internally we'll treat all options as an array
            if ($.isPlainObject(shapes))
                shapes = [shapes];

            // load the library
            self._loadRendererAsync().done(function (renderer) {
                for (var i = 0; i < shapes.length; i++) {
                    var shape = shapes[i];
                    var types = $.extend({}, renderer.types, $.graphicsSettings().types); // get types 
                    if (!self.canvas)
                        renderer.createCanvas(self);
                    if (shape.type in types)
                        types[shape.type].apply(self, [self.canvas, renderer, shape]);
                    else
                        throw "Type " + shape.type + " not found.";
                }
            });
        },
        _loadRendererAsync: function () {
            console.info("loading renderers");
            var renderers = this.settings.renderers.slice(0);
            while (renderers.length > 0) {
                var renderer = renderers.shift();
                if ($.isPlainObject(this.settings._renderers[renderer])) { // if renderer has been cached, load it
                    return $.when(this.settings._renderers[renderer]);
                } else if ($.isPlainObject(renderer)) { // if the renderer is a function, return it directly
                    return $.when(renderer);
                } else if (typeof renderer === "string") { // if the renderer is a string dynamically load it
                    // figure out if the rendere is a url
                    var pathToLoad = renderer;
                    if (renderer.indexOf("/") == -1)
                        pathToLoad = this.settings.basePath + "jqgl." + pathToLoad + ".js";
                    var dfd = $.Deferred();
                    require(["scripts/jqgl/jqgl." + renderer], function (renderer) {
                        dfd.resolve(renderer);
                    });
                    return dfd;
                }
            }
        }
    };

    $.extend($, {
        graphicsSettings: function (options) {
            if (options == undefined) {
                return $._graphicsSettings;
            } else {
                $._graphicsSettings = $.extend({}, new jqgl.baseImplementation(), options);
            }
        },
        _graphicsSettings: $.extend({}, new jqgl.baseImplementation())
    });

    $.fn.graphics = function (settings, shapes) {
        $(this).each(function () {
            if (jqgl.widget[settings]) {
                return jqgl.widget[settings].apply($(this).data("_graphics"), Array.prototype.slice.call(arguments, 1));
            } else if (typeof settings === 'object') {
                if (shapes === undefined) {
                    shapes = settings;
                    settings = $.graphicsSettings();
                } else {
                    settings = $.extend({}, $.graphicsSettings(), settings);
                }
                var self = $.extend(new jqgl.widget(), {
                    element: $(this),
                    settings: settings,
                    shapes: shapes
                }, jqgl.widget);
                $(this).data("_graphics", self);
                return self._create(settings, shapes);
            } else {
                $.error('Method ' + method + ' does not exist on jQuery.graphics');
            }
        });
    };
})(jQuery);